/*
 * Copyright (C) 2010 Jordi Mas i Hernàndez <jmas@softcatala.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, see <http://www.gnu.org/licenses/>.
 */


namespace gbrainy.Core.Main
{
	public enum MouseEventType
	{
		Move,
		ButtonPress,
		DoubleClick,
	};

	public struct MouseEventArgs
	{
		public double X { get; set; }
		public double Y { get; set; }
		public MouseEventType EventType { get; set; }

		public MouseEventArgs (double x, double y, MouseEventType event_type) : this ()
		{
			X = x;
			Y = y;
			EventType = event_type;
		}
	}

	public interface IMouseEvent
	{
		void MouseEvent (object obj, MouseEventArgs args);
	}
}
